/*
 * Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana
 *
 * This file is part of blendb.
 *
 * blendb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * blendb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with blendb.  If not, see <http://www.gnu.org/licenses/>.
 */

const miss = require("mississippi");
const uuid = require("node-uuid");

import { Server } from "./server";
import { Storage } from "./storage";

export interface ISourceOptions {
    tags: string[];
}

export class Source {
    public id: string;
    public tags: string[];
    private server: Server;
    private storage: Storage;
    private data: any[];

    constructor(server: Server, options: ISourceOptions) {
        this.id = uuid.v4();
        this.server = server;
        this.storage = server.storage.l1;
        this.tags = options.tags;
        this.data = new Array();
    }

    public insert(doc: any): void {
        this.data.push({
            doc: doc
        });
    }

    public stream() {
        let i = 0;
        return miss.from.obj((size: number, next: Function) => {
            if (i >= this.data.length) {
                next(null, null);
                return;
            }

            next(null, this.data[i++]);
        });
    }

    // insert(data: any): void {
    //     this.storage.insert({
    //         tags: this.tags,
    //         data: data
    //     });
    // }
}
