#!/usr/bin/env node
/*
 * Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana
 *
 * This file is part of blendb.
 *
 * blendb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * blendb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with blendb.  If not, see <http://www.gnu.org/licenses/>.
 */

// external libraries
import express = require("express");
const osprey = require("osprey");
const ramlParser = require("raml-parser");

// load router
import { router } from "./api/router-v1";

// load server
import { Server } from "./core/server";

// create a new express app
const app = module.exports = express();

const blendb = new Server();
app.locals.blendb = blendb;

////// TESTING ////////
let s1 = blendb.source("s1", { tags: ["telecentro", "v1"] });
let s2 = blendb.source("s2", { tags: ["telecentro", "v2"] });
let s3 = blendb.source("s3", { tags: ["gesac"] });

for (let i = 0; i < 1024; i++) {
    s1.insert({
        id: i % 10,
        seed: ((i % 10) === 0) ? 5 : Math.random()
    });
}
for (let i = 0; i < 256; i++) {
    s2.insert({
        id: i % 5,
        seed: ((i % 5) === 0) ? 26 : Math.random()
    });
}
for (let i = 0; i < 2048; i++) {
    s3.insert({
        id: i % 8 + 5,
        seed: ((i % 8) === 0) ? 13 : Math.random()
    });
}

let position = ((data: any): any => {
    return {
        id: data.id * 2,
        local: data.seed + 1
    };
});
let measure = ((data: any): any => {
    return {
        id: data.id * 3,
        local: data.seed + 2
    };
});
let aggregate = ((data: any, newDimensions: string[]): any => {
    return data;
});

let t1 = blendb.transformer("t1", {
    dimensions: ["dim:one", "dim:two"],
    metrics: ["met:one"],
    operators: {
        position: position,
        measure: measure,
        aggregate: aggregate,
    }
});
let t2 =  blendb.transformer("t2", {
    dimensions: ["dim:one", "dim:three", "dim:four"],
    metrics: ["met:two", "met:three"],
    operators: {
        position: position,
        measure: measure,
        aggregate: aggregate
    }
});
let t3 =  blendb.transformer("t3", {
    dimensions: ["dim:two"],
    metrics: ["met:four", "met:five"],
    operators: {
        position: position,
        measure: measure,
        aggregate: aggregate
    }
});

t1.setSource(s1);
t2.setSource(s2);
t3.setSource(s3);
///////////////////////

app.set("view engine", "pug");
app.set("views", "src/api/views");

// parse the RAML spec and load osprey middleware
ramlParser.loadFile("specs/blendb-api-v1.raml")
    .then((raml: any) => {
        app.use("/v1",
            osprey.security(raml),
            osprey.server(raml),
            router);

        let port = process.env.PORT || 3000;
        app.listen(port);

        if (app.get("env") === "development") {
            console.log("Server listening on port " + port + ".");
        }
    },
    (err: any) => {
        console.error("RAML Parsing Error: " + err.message);
        process.exit(1);
    });
